uniform sampler2D colormap;
uniform sampler2D shadowmap;
uniform float a2v_MeltingAlpha;

varying vec3 v2f_ShadowInfo;

const vec2 TexelSize = vec2(1.0/1024.0, 1.0/1024.0);

float OffsetLockup(vec2 loc, vec2 offset)
{ return texture2D(shadowmap, loc + (offset-vec2(1.5)) * TexelSize).x; }

float ShadowPCF1X1(vec3 shadowinfo)
{ return (shadowinfo.z - OffsetLockup(shadowinfo.xy, vec2(0.0, 0.0)) > 0.0) ? 1.0 : 0.0; }

float ShadowPCF2X2(vec3 shadowinfo)
{
	vec4 Accum;
	Accum.x = shadowinfo.z - OffsetLockup(shadowinfo.xy, vec2(0.0, 0.0));
	Accum.y = shadowinfo.z - OffsetLockup(shadowinfo.xy, vec2(0.0, 1.0));
	Accum.z = shadowinfo.z - OffsetLockup(shadowinfo.xy, vec2(1.0, 0.0));
	Accum.w = shadowinfo.z - OffsetLockup(shadowinfo.xy, vec2(1.0, 1.0));

	Accum.x = (Accum.x > 0.0) ? 1.0 : 0.0;
	Accum.y = (Accum.y > 0.0) ? 1.0 : 0.0;
	Accum.z = (Accum.z > 0.0) ? 1.0 : 0.0;
	Accum.w = (Accum.w > 0.0) ? 1.0 : 0.0;

	return mix(mix(Accum.x, Accum.y, shadowinfo.x),
		mix(Accum.z, Accum.w, shadowinfo.x),
		shadowinfo.y);
}

void main()
{
	float tmp = texture2D(colormap, gl_TexCoord[0].xy).x;
	if (tmp < 0.05)
		discard;
		
	float shadow = ShadowPCF2X2(v2f_ShadowInfo);
	
	gl_FragColor = vec4(tmp) * (shadow*0.7+0.3);
	gl_FragColor.w = a2v_MeltingAlpha;
}
