uniform sampler2D colormap;
uniform sampler2D mirrormap;
uniform sampler2D shadowmap;

uniform float a2v_Time;
uniform float a2v_WinterValue;

varying vec2 v2f_Texcoord;
varying vec3 v2f_ShadowInfo;
varying vec4 v2f_MirrorTexcoord;

const vec2 TexelSize = vec2(1.0/1024.0, 1.0/1024.0);

float OffsetLockup(vec2 loc, vec2 offset)
{ return texture2D(shadowmap, loc + (offset-vec2(1.5)) * TexelSize).x; }

float ShadowPCF1X1(vec3 shadowinfo)
{ return (shadowinfo.z - OffsetLockup(shadowinfo.xy, vec2(0.0, 0.0)) > 0.0) ? 1.0 : 0.0; }

float ShadowPCF2X2(vec3 shadowinfo)
{
	vec4 Accum;
	Accum.x = shadowinfo.z - OffsetLockup(shadowinfo.xy, vec2(0.0, 0.0));
	Accum.y = shadowinfo.z - OffsetLockup(shadowinfo.xy, vec2(0.0, 1.0));
	Accum.z = shadowinfo.z - OffsetLockup(shadowinfo.xy, vec2(1.0, 0.0));
	Accum.w = shadowinfo.z - OffsetLockup(shadowinfo.xy, vec2(1.0, 1.0));

	Accum.x = (Accum.x > 0.0) ? 1.0 : 0.0;
	Accum.y = (Accum.y > 0.0) ? 1.0 : 0.0;
	Accum.z = (Accum.z > 0.0) ? 1.0 : 0.0;
	Accum.w = (Accum.w > 0.0) ? 1.0 : 0.0;

	return mix(mix(Accum.x, Accum.y, shadowinfo.x),
		mix(Accum.z, Accum.w, shadowinfo.x),
		shadowinfo.y);
}

float ShadowPCF4X4(vec3 shadowinfo)
{
vec4 row1;
	row1.x = shadowinfo.z - OffsetLockup(shadowinfo.xy, vec2(0.0, 0.0));
	row1.y = shadowinfo.z - OffsetLockup(shadowinfo.xy, vec2(1.0, 0.0));
	row1.z = shadowinfo.z - OffsetLockup(shadowinfo.xy, vec2(2.0, 0.0));
	row1.w = shadowinfo.z - OffsetLockup(shadowinfo.xy, vec2(3.0, 0.0));

	row1.x = (row1.x > 0.0) ? 1.0 : 0.0;
	row1.y = (row1.y > 0.0) ? 1.0 : 0.0;
	row1.z = (row1.z > 0.0) ? 1.0 : 0.0;
	row1.w = (row1.w > 0.0) ? 1.0 : 0.0;

	vec4 row2;
	row2.x = shadowinfo.z - OffsetLockup(shadowinfo.xy, vec2(0.0, 1.0));
	row2.y = shadowinfo.z - OffsetLockup(shadowinfo.xy, vec2(1.0, 1.0));
	row2.z = shadowinfo.z - OffsetLockup(shadowinfo.xy, vec2(2.0, 1.0));
	row2.w = shadowinfo.z - OffsetLockup(shadowinfo.xy, vec2(3.0, 1.0));

	row2.x = (row2.x > 0.0) ? 1.0 : 0.0;
	row2.y = (row2.y > 0.0) ? 1.0 : 0.0;
	row2.z = (row2.z > 0.0) ? 1.0 : 0.0;
	row2.w = (row2.w > 0.0) ? 1.0 : 0.0;

	vec4 row3;
	row3.x = shadowinfo.z - OffsetLockup(shadowinfo.xy, vec2(0.0, 2.0));
	row3.y = shadowinfo.z - OffsetLockup(shadowinfo.xy, vec2(1.0, 2.0));
	row3.z = shadowinfo.z - OffsetLockup(shadowinfo.xy, vec2(2.0, 2.0));
	row3.w = shadowinfo.z - OffsetLockup(shadowinfo.xy, vec2(3.0, 2.0));

	row3.x = (row3.x > 0.0) ? 1.0 : 0.0;
	row3.y = (row3.y > 0.0) ? 1.0 : 0.0;
	row3.z = (row3.z > 0.0) ? 1.0 : 0.0;
	row3.w = (row3.w > 0.0) ? 1.0 : 0.0;

	vec4 row4;
	row4.x = shadowinfo.z - OffsetLockup(shadowinfo.xy, vec2(0.0, 3.0));
	row4.y = shadowinfo.z - OffsetLockup(shadowinfo.xy, vec2(1.0, 3.0));
	row4.z = shadowinfo.z - OffsetLockup(shadowinfo.xy, vec2(2.0, 3.0));
	row4.w = shadowinfo.z - OffsetLockup(shadowinfo.xy, vec2(3.0, 3.0));

	row4.x = (row4.x > 0.0) ? 1.0 : 0.0;
	row4.y = (row4.y > 0.0) ? 1.0 : 0.0;
	row4.z = (row4.z > 0.0) ? 1.0 : 0.0;
	row4.w = (row4.w > 0.0) ? 1.0 : 0.0;

	return dot((row1 + row2 + row3 + row4) / 16.0, vec4(1.0));
}

float ShadowPCF4X4Jittered(vec3 shadowinfo)
{
	vec4 row1;
	row1.x = shadowinfo.z - OffsetLockup(shadowinfo.xy, 512.0*vec2(-0.000692, -0.000868));
	row1.y = shadowinfo.z - OffsetLockup(shadowinfo.xy, 512.0*vec2(0.000450, -0.002347));
	row1.z = shadowinfo.z - OffsetLockup(shadowinfo.xy, 512.0*vec2(0.000773, -0.002042));
	row1.w = shadowinfo.z - OffsetLockup(shadowinfo.xy, 512.0*vec2(0.001880, -0.001592));

	row1.x = (row1.x > 0.0) ? 1.0 : 0.0;
	row1.y = (row1.y > 0.0) ? 1.0 : 0.0;
	row1.z = (row1.z > 0.0) ? 1.0 : 0.0;
	row1.w = (row1.w > 0.0) ? 1.0 : 0.0;

	vec4 row2;
	row2.x = shadowinfo.z - OffsetLockup(shadowinfo.xy, 512.0*vec2(-0.001208, -0.001198));
	row2.y = shadowinfo.z - OffsetLockup(shadowinfo.xy, 512.0*vec2(-0.000915, -0.000425));
	row2.z = shadowinfo.z - OffsetLockup(shadowinfo.xy, 512.0*vec2(-0.000050, 0.000105));
	row2.w = shadowinfo.z - OffsetLockup(shadowinfo.xy, 512.0*vec2(0.001719, -0.000753));

	row2.x = (row2.x > 0.0) ? 1.0 : 0.0;
	row2.y = (row2.y > 0.0) ? 1.0 : 0.0;
	row2.z = (row2.z > 0.0) ? 1.0 : 0.0;
	row2.w = (row2.w > 0.0) ? 1.0 : 0.0;

	vec4 row3;
	row3.x = shadowinfo.z - OffsetLockup(shadowinfo.xy, 512.0*vec2(-0.001855, -0.000004));
	row3.y = shadowinfo.z - OffsetLockup(shadowinfo.xy, 512.0*vec2(-0.001212, 0.001140));
	row3.z = shadowinfo.z - OffsetLockup(shadowinfo.xy, 512.0*vec2(0.000684, 0.000273));
	row3.w = shadowinfo.z - OffsetLockup(shadowinfo.xy, 512.0*vec2(0.000647, 0.000177));

	row3.x = (row3.x > 0.0) ? 1.0 : 0.0;
	row3.y = (row3.y > 0.0) ? 1.0 : 0.0;
	row3.z = (row3.z > 0.0) ? 1.0 : 0.0;
	row3.w = (row3.w > 0.0) ? 1.0 : 0.0;

	vec4 row4;
	row4.x = shadowinfo.z - OffsetLockup(shadowinfo.xy, 512.0*vec2(-0.001448, 0.002095));
	row4.y = shadowinfo.z - OffsetLockup(shadowinfo.xy, 512.0*vec2(0.000421, 0.000811));
	row4.z = shadowinfo.z - OffsetLockup(shadowinfo.xy, 512.0*vec2(0.000542, 0.001491));
	row4.w = shadowinfo.z - OffsetLockup(shadowinfo.xy, 512.0*vec2(0.002367, 0.000537));

	row4.x = (row4.x > 0.0) ? 1.0 : 0.0;
	row4.y = (row4.y > 0.0) ? 1.0 : 0.0;
	row4.z = (row4.z > 0.0) ? 1.0 : 0.0;
	row4.w = (row4.w > 0.0) ? 1.0 : 0.0;

	return dot((row1 + row2 + row3 + row4) / 16.0, vec4(1.0));
}

void main()
{
	float shadow = ShadowPCF4X4Jittered(v2f_ShadowInfo);
	
	float time = abs(0.5 - fract(a2v_Time * 0.1)) * 100.0;
	vec2 offset = vec2(sin(v2f_Texcoord.x*time), cos(v2f_Texcoord.y*time))*0.001*a2v_WinterValue;
	vec2 tmp = v2f_MirrorTexcoord.xy/v2f_MirrorTexcoord.w + offset;
	float alpha = 0.5*a2v_WinterValue + 0.8*(1.0-a2v_WinterValue);
	vec4 water_color = texture2D(colormap, (v2f_Texcoord + offset)*4.0) * alpha + texture2D(mirrormap, tmp)*(1.0-alpha);
	
	gl_FragColor = water_color*(shadow*0.7+0.3) * (1.0 - 0.3*a2v_WinterValue);
	gl_FragColor.w = 0.9*a2v_WinterValue + 1.0*(1-a2v_WinterValue);
}
